/*
 * Handle all background script events, passing them as
 * appropriate to the webpage.
 */

/*
 * Firefox
 * =======
 *
 * We cannot document.dispatchEvent on Firefox. Instead, declare a crypto.signText function
 * using exportFunction to pass the messages.
 */
if (typeof exportFunction == 'function') {

	function uuidv4() {
		return ([1e7] + -1e3 + -4e3 + -8e3 + -1e11).replace(/[018]/g, c =>
			(c ^ crypto.getRandomValues(new Uint8Array(1))[0] & 15 >> c / 4).toString(16)
		);
	}

	signText = function SignText(source, options, ...CAs) {

		const uuid = uuidv4();

console.log("SignText: ");

		if (typeof source === 'undefined') {

			/*
			 * We've been asked for info about the signtext extension.
			 *
			 * This can be used to detect support for the extension
			 * before trying to use it.
			 */

			const manifest = browser.runtime.getManifest();
			const response = cloneInto({
				name: manifest.name,
				version: manifest.version
			}, window.wrappedJSObject);

			return new window.wrappedJSObject.Promise(
				exportFunction(
					function(resolve, reject) {
						resolve(response);
					}, window.wrappedJSObject
				)
			);

		}

		const port = browser.runtime.connect('rst@redwax.eu', {
			name: window.location.href
		});

		/* pass a string, we return a string */
		if (typeof source === 'string') {

			var result = [];

			/* take promise functions outside for later callback */

			var stringResolve;
			var stringReject;

			var promise = new window.wrappedJSObject.Promise(
				exportFunction(
					function(resolve, reject) {
						stringResolve = resolve;
						stringReject = reject;
					}, window.wrappedJSObject
				)
			);

			const signTextHandler = function(e) {
				/* message not for us */
				if ((typeof e.uuid === "undefined") || (e.uuid !== uuid)) {
					/* do nothing */
				}
				/* an error was received, cancel the stream */
				else if (typeof e.error !== "undefined") {
					const error = new window.wrappedJSObject.Error(e.error);
					stringReject(error);
					port.onMessage.removeListener(signTextHandler);
				}
				/* no response, weird, throw an error back */
				else if (typeof e.response === "undefined") {
					stringReject(new Error("error:noResponse"));
					port.onMessage.removeListener(signTextHandler);
				}
				/* an ack was received, send the next chunk */
				else if (e.response === true) {
					port.postMessage({
						protocol: location.protocol,
						hostname: location.hostname,
						contentType: 'text/plain',
						uuid: uuid,
						request: {
							signText: options,
							CAs: CAs
						}
					});
				}
				/* an eos was received, we're done */
				else if (e.response === false) {
					stringResolve(result.join(""));
					port.onMessage.removeListener(signTextHandler);
				}
				/* a response was received, send ack */
				else {
					result.push(e.response);
					port.postMessage({
						protocol: location.protocol,
						hostname: location.hostname,
						contentType: 'text/plain',
						uuid: uuid,
						request: true
					});
				}
			};

			/* add our listener */
			port.onMessage.addListener(signTextHandler);

			port.onDisconnect.addListener((p) => {
				if (p.error) {
					console.log(`Redwax SignText: disconnected due to an error: ${p.error.message}`);
				} else {
					console.log(`Redwax SignText: disconnected`);
				}
			});


			/* fire off our initial message */
			port.postMessage({
				protocol: location.protocol,
				hostname: location.hostname,
				contentType: 'text/plain',
				uuid: uuid,
				request: source
			});

			return promise;
		}

		else {

			const response = new window.wrappedJSObject.Error('error:unknownType');

			return new window.wrappedJSObject.Promise(
				exportFunction(
					function(resolve, reject) {
						reject(response);
					}, window.wrappedJSObject
				)
			);
		}

	};

	exportFunction(signText, window.crypto, {
		defineAs: "signText"
	});

}

